<?php
namespace Ak\Orm;

use Ak\Container\AbstractFactory\PluginManagerAbstractFactory;
use Ak\Orm\Mapping\DiscriminatorSubscriber;
use Ak\Orm\Service\AnnotationReaderAwareInitializer;
use Doctrine\Common\Annotations\Reader;
use Doctrine\Persistence\Mapping\Driver\MappingDriver;
use Doctrine\ORM\EntityManagerInterface;
use Gedmo\Timestampable\TimestampableListener;
use Laminas\ServiceManager\AbstractFactory\ReflectionBasedAbstractFactory;
use Laminas\ServiceManager\Factory\InvokableFactory;

return [
    'dependencies' => [
        'aliases' => [
            'em'                     => EntityManagerInterface::class,
            'orm.em'                 => EntityManagerInterface::class,
            'orm.annotation_reader'  => Reader::class,
            'orm.mapping_driver'     => MappingDriver::class,
            'orm.repository_factory' => Service\RepositoryFactory::class,
        ],

        'factories' => [
            'orm.configuration'           => Service\ConfigurationFactory::class,
            'orm.cache'                   => Service\CacheFactory::class,
            'orm.events'                  => Service\EventManagerFactory::class,

            DiscriminatorSubscriber::class => ReflectionBasedAbstractFactory::class,
            TimestampableListener::class => InvokableFactory::class,

            Reader::class                 => Service\AnnotationReaderFactory::class,
            MappingDriver::class          => Service\MappingDriverFactory::class,
            EntityManagerInterface::class => Service\EntityManagerFactory::class,
            Service\RepositoryPluginManager::class => PluginManagerAbstractFactory::class,
            Middleware\EntityManagerMiddleware::class => Middleware\EntityManagerMiddlewareFactory::class,
        ],

        'initializers' => [
            AnnotationReaderAwareInitializer::class,
        ],

        'abstract_factories' => [
            ReflectionBasedAbstractFactory::class,
        ],
    ],
];
