<?php
namespace Ak\Orm\Form\Element;

use Ak\Orm\Service\EntityManagerAwareInterface;
use Ak\Orm\Service\EntityManagerAwareTrait;
use Laminas\Form\Element\MultiCheckbox;

/**
 * Class EntityMulticheckbox
 *
 * @package Ak\Orm\Form\Element
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
class EntityMulticheckbox extends MultiCheckbox implements EntityManagerAwareInterface
{
    use EntityManagerAwareTrait;

    /** @var array */
    protected $valueOptions;

    /**
     * @return array
     */
    public function getValueOptions()
    {
        if (null === $this->valueOptions) {
            $this->valueOptions = [];
            $entity = $this->getOption('entity');
            $valueField = $this->getOption('value_field');

            if (empty($entity)) {
                throw new \RuntimeException('Postavka `entity` je obavezna.');
            }

            if (!class_exists($entity)) {
                throw new \RuntimeException(sprintf(
                    "Entitet '%s' ne postoji.", $entity
                ));
            }

            if (empty($valueField)) {
                throw new \RuntimeException('Postavka `value_field` je obavezna.');
            }

            $query = $this->em->createQueryBuilder()
                ->select(['e.id', sprintf('e.%s', $valueField)])
                ->from($entity, 'e')
                ->getQuery();

            foreach ($query->getResult() as $row) {
                $this->valueOptions[intval($row['id'])] = $row[$valueField];
            }

            asort($this->valueOptions);
        }

        return $this->valueOptions;
    }
}
