<?php
namespace Ak\Orm\Hydrator;

use Laminas\Hydrator\ReflectionHydrator;

/**
 * Class HydratedObjectHydrator
 *
 * @package Ak\Orm\Hydrator
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class HydratedObjectHydrator extends ReflectionHydrator
{
    public function extract(object $object, int $depth = 0): array
    {
        $result = [];

        foreach (self::getReflProperties($object) as $property) {
            $propertyName = $this->extractName($property->getName(), $object);

            if (! $this->getCompositeFilter()->filter($propertyName)) {
                continue;
            }

            $value = $property->getValue($object);
            $result[$propertyName] = $this->extractValue($propertyName, $value, $object);

            if (is_object($result[$propertyName]) && $depth < 2) {
                $result[$propertyName] = $this->extract($result[$propertyName], $depth + 1);
            }
        }

        return $result;
    }

    public function hydrate(array $data, object $object)
    {
        $reflProperties = self::getReflProperties($object);

        foreach ($data as $key => $value) {
            $name = $this->hydrateName($key, $data);

            if (isset($reflProperties[$name])) {
                if (is_array($value) && is_object($current = $reflProperties[$name]->getValue($object))) {
                    $currentValue = $this->extract($current);

                    if ($currentValue !== $value) {
                        $current = clone $current;
                    }

                    $current = $this->hydrate($value, $current);
                    $reflProperties[$name]->setValue($object, $current);
                } else {
                    $reflProperties[$name]->setValue($object, $this->hydrateValue($name, $value, $data));
                }
            }
        }

        return $object;
    }
}
