<?php
namespace Ak\Orm\Service;

use Doctrine\Common\Cache\ArrayCache;
use Doctrine\Common\Cache\CacheProvider;
use Doctrine\Common\Cache\FilesystemCache;
use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Factory\FactoryInterface;

/**
 * Stvara novu instancu keša meta-podataka.
 *
 * @package Ak\Orm\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class CacheFactory implements FactoryInterface
{
    /**
     * Stvara keš.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return CacheProvider
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $container->get('config');

        if ($config['development'] ?? false) {
            return new ArrayCache();
        }

        $config = $config['orm']['cache'] ?? [];

        switch ($config['name'] ?? null) {
            case 'filesystem':
                $options = $config['options'] ?? ['directory' => sys_get_temp_dir()];

                if (!isset($options['directory']) || empty($options['directory'])) {
                    throw new ServiceNotCreatedException('Direktorij do keša meta-podataka nije postavljen.');
                }

                return new FilesystemCache(...(array_values($config['options'] ?? [sys_get_temp_dir()])));
                break;
            default:
                return new ArrayCache();
        }
    }
}
