<?php
namespace Ak\Orm\Service;

use Doctrine\ORM\Configuration;
use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Factory\FactoryInterface;

/**
 * Stvara novu instancu konfiguracije entity managera.
 *
 * @package Ak\Orm\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class ConfigurationFactory implements FactoryInterface
{
    /**
     * Stvara konfiguraciju.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return Configuration
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $container->get('config') ?? [];
        $isDevelopment = $config['development'] ?? false;
        $config = $config['orm'] ?? [];

        $cache = $container->get('orm.cache');
        $driver = $container->get('orm.mapping_driver');
        $repoFactory = $container->get('orm.repository_factory');

        $configuration = new Configuration();
        $configuration->setMetadataCacheImpl($cache);
        $configuration->setQueryCacheImpl($cache);
        $configuration->setResultCacheImpl($cache);
        $configuration->setProxyDir($config['proxy_dir'] ?? null);
        $configuration->setProxyNamespace('Ak\Orm\Proxy');
        $configuration->setAutoGenerateProxyClasses($isDevelopment);
        $configuration->setMetadataDriverImpl($driver);
        $configuration->setRepositoryFactory($repoFactory);

        return $configuration;
    }
}
