<?php
namespace Ak\Orm\Service;

use Doctrine\Common\EventManager;
use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Factory\FactoryInterface;

/**
 * Class EventManagerFactory
 *
 * @package Ak\Orm\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class EventManagerFactory implements FactoryInterface
{
    /**
     * Create an object
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return EventManager
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $container->get('config')['orm'] ?? [];
        $events = $container->get('db.events');

        foreach ($config['event_subscribers'] ?? [] as $subscriber) {
            if (is_string($subscriber)) {
                $subscriber = $container->get($subscriber);
            }

            $events->addEventSubscriber($subscriber);
        }

        return $events;
    }
}
