<?php
namespace Ak\Orm\Service;

use Doctrine\Persistence\ObjectRepository;
use Doctrine\ORM\EntityManagerInterface;
use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Factory\AbstractFactoryInterface;

/**
 * Class RepositoryAbstractFactory
 *
 * @package Ak\Orm\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class RepositoryAbstractFactory implements AbstractFactoryInterface
{
    /**
     * Can the factory create an instance for the service?
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @return bool
     */
    public function canCreate(ContainerInterface $container, $requestedName)
    {
        $interfaces = class_implements($requestedName);

        if (!in_array(ObjectRepository::class, $interfaces)) {
            $repository = $this->getRepositoryClass($container, $requestedName);
            return $this->canCreate($container, $repository);
        }

        return in_array(ObjectRepository::class, $interfaces);
    }

    /**
     * Create an object
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return ObjectRepository
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        /** @var EntityManagerInterface $em */
        $em = $container->get(EntityManagerInterface::class);
        $metadata = $em->getClassMetadata($requestedName);

        $repositoryClassName = $this->getRepositoryClass($container, $requestedName);

        return new $repositoryClassName($em, $metadata);
    }

    /**
     * @param ContainerInterface $container
     * @param string $entityClass
     * @return string
     */
    private function getRepositoryClass(ContainerInterface $container, string $entityClass): string
    {
        /** @var EntityManagerInterface $em */
        $em = $container->get(EntityManagerInterface::class);
        $metadata = $em->getClassMetadata($entityClass);

        return $metadata->customRepositoryClassName ?: $em->getConfiguration()->getDefaultRepositoryClassName();
    }
}
