<?php
namespace Ak\Orm\Service;

use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Repository\RepositoryFactory as RepositoryFactoryInterface;

/**
 * Class RepositoryFactory
 *
 * @package Ak\Orm\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class RepositoryFactory implements RepositoryFactoryInterface
{
    /** @var RepositoryPluginManager */
    private $plugins;

    private $repos = [];

    /**
     * RepositoryFactory constructor.
     * @param RepositoryPluginManager $plugins
     */
    public function __construct(RepositoryPluginManager $plugins)
    {
        $this->plugins = $plugins;
    }

    /**
     * Gets the repository for an entity class.
     *
     * @param \Doctrine\ORM\EntityManagerInterface $entityManager The EntityManager instance.
     * @param string $entityName The name of the entity.
     *
     * @return \Doctrine\Common\Persistence\ObjectRepository
     */
    public function getRepository(EntityManagerInterface $entityManager, $entityName)
    {
        $hash = $entityManager->getClassMetadata($entityName)->getName() . spl_object_hash($entityManager);

        if (isset($this->repos[$hash])) {
            return $this->repos[$hash];
        }

        return $this->repos[$hash] = $this->plugins->get($entityName);
    }
}