<?php
namespace AkTest\Orm;

use Ak\Form\FormElementManager;
use Ak\Orm\ConfigProvider;
use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Tools\Console\ConsoleRunner;
use Symfony\Component\Console\Input\ArrayInput;
use Symfony\Component\Console\Output\NullOutput;

/**
 * Class AbstractTestCase
 *
 * @package AkTest\Orm
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
abstract class AbstractTestCase extends \Ak\Test\AbstractTestCase
{
    protected $providers = [
        \Ak\InputFilter\ConfigProvider::class,
        ConfigProvider::class,
        TestConfigProvider::class,
    ];

    /**
     * @throws \Exception
     */
    protected function setUp()
    {
        parent::setUp();

        $em = $this->getEntityManager();

        $helperSet = ConsoleRunner::createHelperSet($em);
        $cli = ConsoleRunner::createApplication($helperSet, []);
        $cli->setAutoExit(false);
        $cli->setCatchExceptions(false);

        $cli->run(new ArrayInput(['orm:schema-tool:drop', '--force' => true]), new NullOutput());
        $cli->run(new ArrayInput(['orm:schema-tool:create']), new NullOutput());
    }

    /**
     * @return EntityManagerInterface|EntityManager
     */
    protected function getEntityManager(): EntityManagerInterface
    {
        $container = $this->getContainer();
        return $container->get(EntityManagerInterface::class);
    }

    /**
     * @return Connection
     */
    protected function getConnection(): Connection
    {
        $em = $this->getEntityManager();
        return $em->getConnection();
    }

    /**
     * @return FormElementManager
     */
    protected function getFormElementManager(): FormElementManager
    {
        $container = $this->getContainer();
        return $container->get(FormElementManager::class);
    }
}