<?php
namespace AkTest\Orm\Entity;

use AkCustomer\Entity\Customer;
use AkEmployee\Entity\Developer;
use AkTest\Orm\AbstractTestCase;

final class SIngleTableInheritanceTest extends AbstractTestCase
{
    /**
     * @throws \Doctrine\DBAL\DBALException
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function testCanSaveParentEntities()
    {
        $em = $this->getEntityManager();
        $person = new Person();

        $em->persist($person);
        $em->flush();

        $conn = $em->getConnection();
        /** @noinspection PhpUnhandledExceptionInspection */
        $query = $conn->query('SELECT * FROM person');
        $rows = $query->fetchAll();

        $this->assertCount(1, $rows);
        $this->assertEquals(1, $rows[0]['id']);
        $this->assertEquals('person', $rows[0]['type']);
    }

    /**
     * @throws \Doctrine\DBAL\DBALException
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function testCanSaveChildEntities()
    {
        $em = $this->getEntityManager();
        $person = new Employee();

        $em->persist($person);
        $em->flush();

        $conn = $em->getConnection();
        /** @noinspection PhpUnhandledExceptionInspection */
        $query = $conn->query('SELECT * FROM person');
        $rows = $query->fetchAll();

        $this->assertCount(1, $rows);
        $this->assertEquals(1, $rows[0]['id']);
        $this->assertEquals('person_employee', $rows[0]['type']);
    }

    /**
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testCanWorkWithInheritanceAcrossNamespaces()
    {
        $em = $this->getEntityManager();
        $customer = new Customer();

        $em->persist($customer);
        $em->flush();

        $conn = $this->getConnection();
        $row = $conn->query('SELECT * FROM person WHERE id = 1')->fetch();

        $this->assertEquals('customer', $row['type']);

        $entity = $em->find(Person::class, 1);

        $this->assertInstanceOf(Customer::class, $entity);
    }

    /**
     * @throws \Doctrine\DBAL\DBALException
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     * @throws \Doctrine\ORM\TransactionRequiredException
     */
    public function testCanWorkWithInheritanceAcrossMultipleNamespaces()
    {
        $em = $this->getEntityManager();
        $developer = new Developer();

        $em->persist($developer);
        $em->flush();

        $conn = $this->getConnection();
        $row = $conn->query('SELECT * FROM person WHERE id = 1')->fetch();

        $this->assertEquals('developer', $row['type']);

        $entity = $em->find(Person::class, 1);

        $this->assertInstanceOf(Developer::class, $entity);
    }
}