<?php
namespace AkTest\Orm\Form;

use Ak\Form\Annotation\AnnotationBuilder;
use Ak\Form\Element\Submit;
use Ak\Orm\Form\EntityForm;
use Ak\Orm\Hydrator\HydratedObjectHydrator;
use AkTest\Orm\AbstractTestCase;
use AkTest\Orm\Entity\ConcreteEntity;
use AkTest\Orm\Entity\TimestampableEntity;
use PHPUnit\Framework\MockObject\MockObject;
use Laminas\Form\Element\Text;
use Laminas\Form\Form;
use Laminas\Hydrator\HydratorInterface;

/**
 * Class EntityFormTest
 *
 * @package AkTest\Orm\Form
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class EntityFormTest extends AbstractTestCase
{
    public function testIsRegistered()
    {
        $forms = $this->getFormElementManager();
        $this->assertContainerHasService($forms, EntityForm::class, EntityForm::class, false);
    }

    public function testHasTheCorrectHydrator()
    {
        $forms = $this->getFormElementManager();
        /** @var EntityForm $form */
        $form = $forms->get(EntityForm::class);
        $this->assertInstanceOf(HydratedObjectHydrator::class, $form->getHydrator());
    }

    public function testHasSubmitButton()
    {
        $forms = $this->getFormElementManager();
        /** @var Form $form */
        $form = $forms->get(EntityForm::class);

        $form->add(new Text('some_name'));
        /** @noinspection PhpStatementHasEmptyBodyInspection */
        foreach ($form as $last);

        /** @noinspection PhpUndefinedVariableInspection */
        $this->assertInstanceOf(Submit::class, $last);
    }

    public function testWillAddEntityFieldsOnBind()
    {
        $forms = $this->getFormElementManager();
        /** @var Form $form */
        $form = $forms->get(ConcreteEntity::class);
        $form->bind(new ConcreteEntity());

        $form->setBindOnValidate(true);
        $form->bindValues(['name' => 'Some name']);

        /** @var ConcreteEntity $entity */
        $entity = $form->getObject();

        $this->assertEquals('Some name', $entity->getName());
    }

    public function testCanUseHydratorFromEntity()
    {
        /** @var HydratorInterface|MockObject $hydrator */
        $hydrator = $this->createMock(HydratorInterface::class);
        $entity = new ConcreteEntity();
        $entity->setHydrator($hydrator);

        $forms = $this->getFormElementManager();
        /** @var Form $form */
        $form = $forms->get(EntityForm::class);
        $form->bind($entity);

        $this->assertSame($hydrator, $form->getHydrator());
    }

    public function testCanCreateFormFromEntity()
    {
        $container = $this->getContainer();
        /** @var AnnotationBuilder $builder */
        $builder = $container->get('FormAnnotationBuilder');
        $form = $builder->createForm(ConcreteEntity::class);

        $this->assertInstanceOf(EntityForm::class, $form);
    }

    public function testThereAreNoFieldsForTimestampable()
    {
        $forms = $this->getFormElementManager();
        /** @var Form $form */
        $form = $forms->get(TimestampableEntity::class);

        $this->assertFalse($form->has('createdAt'));
        $this->assertFalse($form->has('updatedAt'));
    }
}
