<?php
namespace AkTest\Orm\Service;

use Ak\Orm\ConfigProvider;
use Ak\Orm\Service\CacheFactory;
use Ak\Test\AbstractTestCase;
use AkTest\Orm\TestConfigProvider;
use Doctrine\Common\Cache\ArrayCache;
use Doctrine\Common\Cache\CacheProvider;
use Doctrine\Common\Cache\FilesystemCache;

/**
 * Class MetadataCacheFactoryTest
 *
 * @package AkTest\Orm\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class CacheTest extends AbstractTestCase
{
    protected $providers = [
        ConfigProvider::class,
        TestConfigProvider::class,
    ];

    public function testIsRegistered()
    {
        $container = $this->getContainer();

        $this->assertContainerHasService($container, 'orm.cache', CacheProvider::class);
    }

    public function testWillCreateArrayCacheWhenInDevelopmentMode()
    {
        $container = $this->getContainer([
            'development' => true,
        ]);
        $factory = new CacheFactory();

        $cache = $factory($container, '');

        $this->assertInstanceOf(ArrayCache::class, $cache);
    }

    /**
     * @expectedException \Laminas\ServiceManager\Exception\ServiceNotCreatedException
     */
    public function testWillThrowExceptionIfFilesystemCacheDirectoryNotSet()
    {
        $container = $this->getContainer([
            'development' => false,

            'orm' => [
                'cache' => [
                    'options' => [
                        'directory' => null,
                    ],
                ],
            ],
        ]);
        $factory = new CacheFactory();

        $factory($container, '');
    }

    public function testCanCreateCache()
    {
        $container = $this->getContainer([
            'development' => false,
        ]);
        $factory = new CacheFactory();

        $cache = $factory($container, '');

        $this->assertInstanceOf(FilesystemCache::class, $cache);
    }

    public function testCanCreateCacheWhenNoneConfigured()
    {
        $container = $this->getContainer([
            'orm' => [
                'metadata_cache' => null,
            ],
        ]);
        $factory = new CacheFactory();

        $cache = $factory($container, '');

        $this->assertInstanceOf(CacheProvider::class, $cache);
    }
}
