<?php
namespace AkTest\Orm\Service;

use Ak\Orm\ConfigProvider;
use Ak\Orm\Service\EntityManagerFactory;
use Ak\Test\AbstractTestCase;
use Doctrine\Common\Cache\ArrayCache;
use Doctrine\Common\EventManager;
use Doctrine\DBAL\Connection;
use Doctrine\ORM\Configuration;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityManagerInterface;
use PHPUnit\Framework\MockObject\MockObject;

/**
 * Class EntityManagerFactoryTest
 *
 * @package AkTest\Orm\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class EntityManagerFactoryTest extends AbstractTestCase
{
    protected $providers = [
        ConfigProvider::class,
    ];

    /** @var Connection|MockObject */
    private $db;
    /** @var EventManager */
    private $events;

    /**
     * @inheritdoc
     */
    public function getConfig(array $config = []): array
    {
        $config = parent::getConfig($config);
        $dependencies = &$config['dependencies'];

        if (!isset($dependencies['services']['db'])) {
            $this->events = new EventManager();

            $db = $this->createMock(Connection::class);
            $db->method('getEventManager')->willReturn($this->events);

            $dependencies['services']['db'] = $db;
            $dependencies['services']['db.events'] = $this->events;
        }

        $this->db = $dependencies['services']['db'];

        return $config;
    }

    public function testIsRegistered()
    {
        $container = $this->getContainer();

        $this->assertContainerHasService($container, [
            'em',
            'orm.em',
            EntityManagerInterface::class,
        ], EntityManagerInterface::class, true);
    }

    /**
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testCanCreateTheManager()
    {
        $container = $this->getContainer([
            'development' => true,
        ]);
        $factory = new EntityManagerFactory();

        /** @var EntityManager $em */
        $em = $factory($container, '');

        $this->assertInstanceOf(EntityManagerInterface::class, $em);
        $this->assertSame($this->db, $em->getConnection());
        $this->assertSame($this->events, $em->getEventManager());
        $this->assertInstanceOf(ArrayCache::class, $em->getConfiguration()->getMetadataCacheImpl());
    }

    /**
     * @expectedException \Laminas\ServiceManager\Exception\ServiceNotCreatedException
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testWillCatchORMExceptions()
    {
        $configuration = $this->createMock(Configuration::class);
        $configuration->method('getMetadataDriverImpl')->willReturn(null);

        $container = $this->getContainer([
            'dependencies' => [
                'services' => [
                    'orm.configuration' => $configuration,
                ],
            ],
        ]);
        $factory = new EntityManagerFactory();

        $factory($container, '');
    }
}
