<?php
namespace AkTest\Orm\Type;

use AkTest\Orm\AbstractTestCase;
use Laminas\Json\Json;

/**
 * Class HydratedObjectTest
 *
 * @package AkTest\Orm\Type
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class HydratedObjectTest extends AbstractTestCase
{
    /**
     * @throws \Doctrine\DBAL\DBALException
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function testCanPersistEntity()
    {
        $entity = new HydratedObjectEntity(new HydratedObjectParams('param name', [
            'first' => 'value 1',
            'second' => 'value 2',
        ]));

        $em = $this->getEntityManager();
        $em->persist($entity);
        $em->flush();

        $conn = $this->getConnection();
        $row = $conn->query('SELECT * FROM hydrated_object WHERE id = 1')->fetch();

        $expected = [
            'class' => HydratedObjectParams::class,
            'data' => [
                'name' => 'param name',
                'data' => [
                    'first' => 'value 1',
                    'second' => 'value 2',
                ],
            ],
        ];

        $this->assertEquals($expected, Json::decode($row['params'], Json::TYPE_ARRAY));
    }

    /**
     * @throws \Doctrine\DBAL\DBALException
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     * @throws \Doctrine\ORM\TransactionRequiredException
     */
    public function testCanReadEntity()
    {
        $conn = $this->getConnection();
        $conn->insert('hydrated_object', [
            'id' => 1,
            'params' => Json::encode([
                'class' => HydratedObjectParams::class,
                'data' => [
                    'name' => 'param name',
                    'data' => [
                        'first' => 'value 1',
                        'second' => 'value 2',
                    ],
                ],
            ]),
        ]);

        $em = $this->getEntityManager();
        /** @var HydratedObjectEntity $entity */
        $entity = $em->find(HydratedObjectEntity::class, 1);
        $params = $entity->getParams();

        $this->assertEquals('param name', $params->getName());
        $this->assertEquals([
            'first' => 'value 1',
            'second' => 'value 2',
        ], $params->getData());
    }
}
